@@ -11,6 +11,8 @@ class PackageId(IntEnum):
     RECENTER = 7
     DISPLAY_FRAME = 8
     RAW_DATA = 9
+    SPECTROGRAM = 10
+    MODIFY_API_CONFIG = 11
 
 
 class PackageType(IntEnum):
@@ -34,6 +36,12 @@ class ButtonId(IntEnum):
     BOTTOM = 2
 
 
+class DatastreamType(IntEnum):
+    NONE = 0
+    MOTION_DATA = 1
+    RAW_DATA = 2
+
+
 class ButtonAction(IntEnum):
     """The action of a button package
 
